# Generated by Django 3.0.5 on 2020-06-17 17:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_obm', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='address',
            name='value',
            field=models.CharField(max_length=500, null=True),
        ),
    ]



# Copyright 2019-2020 Alexander Polishchuk
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from django.conf import settings
from rest_framework import serializers

from django_obm import models


class CurrencySerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Currency
        fields = "__all__"


class TransactionSerializer(serializers.ModelSerializer):
    from_address = serializers.CharField(required=False)
    to_address = serializers.CharField(required=True)
    password = serializers.CharField(required=False)
    currency = serializers.SlugRelatedField(
        slug_field="name", queryset=models.Currency.objects.all(),
    )
    subtract_fee_from_amount = serializers.BooleanField(
        required=False,
        default=getattr(
            settings,
            "OBM_REST_SUBTRACT_TRANSACTION_FEE_FROM_AMOUNT_DEFAULT",
            False,
        ),
    )

    class Meta:
        model = models.Transaction
        read_only_fields = [
            "txid",
            "category",
            "fee",
            "timestamp",
        ]
        fields = [
            "password",
            "currency",
            "from_address",
            "to_address",
            "txid",
            "category",
            "amount",
            "block_number",
            "fee",
            "timestamp",
            "subtract_fee_from_amount",
        ]

    def validate(self, attrs):
        currency = attrs.pop("currency")

        # TODO: Add filter with default
        node = models.Node.objects.filter(
            currency=currency,
            is_default=True,
        ).first()
        if not node:
            raise serializers.ValidationError(
                f"Node for {currency.name} does not registered."
            )

        # TODO: Validate for ethereum
        from_address_value = attrs.pop("from_address", None)
        password = attrs.pop("password", "")
        if from_address_value:
            from_address, _ = models.Address.objects.get_or_create(
                value=from_address_value,
                currency=currency,
                defaults={"password": password},
            )
        else:
            from_address = None
        to_address, _ = models.Address.objects.get_or_create(
            value=attrs.pop("to_address"), currency=currency,
        )

        return {
            "node": node,
            "from_address": from_address,
            "to_address": to_address,
            "category": "send",
            **attrs,
        }

    def create(self, validated_data):
        subtract_fee_from_amount = validated_data.pop(
            "subtract_fee_from_amount"
        )
        tx = models.Transaction(**validated_data)
        return tx.send(subtract_fee_from_amount=subtract_fee_from_amount)


class AddressSerializer(serializers.ModelSerializer):
    currency = serializers.SlugRelatedField(
        slug_field="name", queryset=models.Currency.objects.all(),
    )

    class Meta:
        model = models.Address
        read_only_fields = ["value"]
        fields = "__all__"

    def create(self, validated_data):
        currency = validated_data.pop("currency")
        node = models.Node.objects.filter(
            currency=currency,
            is_default=True,
        ).first()
        if not node:
            raise serializers.ValidationError(
                f"Node for {currency.name} does not registered"
            )
        with node:
            address_value = node.create_address()
        return models.Address.objects.create(
            value=address_value,
            currency=currency,
        )


# pylint: disable=abstract-method
class CurrencyEstimateFeeSerializer(serializers.Serializer):
    currency = serializers.CharField(required=True)

    def validate(self, attrs):
        currency_name = attrs["currency"]
        currency = models.Currency.objects \
            .filter(name=currency_name) \
            .first()
        if not currency:
            raise serializers.ValidationError(
                f"Currency '{currency.name}'' does not exist."
            )
        attrs["currency"] = currency
        return attrs



# Generated by Django 3.0.5 on 2020-04-21 08:22

import django.db.models.deletion
import obm.mixins
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Currency',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
            ],
            options={
                'verbose_name_plural': 'currencies',
            },
        ),
        migrations.CreateModel(
            name='Node',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, unique=True)),
                ('is_default', models.BooleanField(default=True, help_text='If True the node will be used as default for transaction sending.')),
                ('rpc_username', models.CharField(help_text='Username for JSON-RPC connections.', max_length=200, verbose_name='RPC username')),
                ('rpc_password', models.CharField(help_text='Password for JSON-RPC connections.', max_length=200, verbose_name='RPC password')),
                ('rpc_host', models.URLField(default='127.0.0.1', help_text='Listen for JSON-RPC connections on this IP address.', verbose_name='RPC host')),
                ('rpc_port', models.PositiveIntegerField(help_text='Listen for JSON-RPC connections on this port.', verbose_name='RPC port')),
                ('timeout', models.FloatField(default=300, help_text='Timeout for call of node JSON RPC.')),
                ('currency', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='nodes', related_query_name='node', to='django_obm.Currency')),
            ],
            options={
                'unique_together': {('rpc_host', 'rpc_port')},
            },
            bases=(models.Model, obm.mixins.NodeMixin),
        ),
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=500)),
                ('password', models.CharField(default='', max_length=500)),
                ('currency', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='addresses', related_query_name='address', to='django_obm.Currency')),
            ],
            options={
                'unique_together': {('value', 'currency')},
            },
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('txid', models.CharField(max_length=500, null=True, verbose_name='transaction id')),
                ('category', models.CharField(max_length=30)),
                ('amount', models.DecimalField(decimal_places=10, help_text='The transaction amount in currency.', max_digits=19)),
                ('block_number', models.PositiveIntegerField(help_text='Number of block that contain the transaction.', null=True)),
                ('fee', models.DecimalField(decimal_places=10, help_text="The amount of the fee in currency. This is only available for the 'send' category of transactions.", max_digits=19, null=True)),
                ('timestamp', models.PositiveIntegerField(help_text='Transaction creation or detection timestamp.', null=True)),
                ('from_address', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='django_obm.Address')),
                ('node', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', related_query_name='transaction', to='django_obm.Node')),
                ('to_address', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', related_query_name='transaction', to='django_obm.Address')),
            ],
            options={
                'unique_together': {('node', 'txid')},
            },
        ),
    ]



# Copyright 2019-2020 Alexander Polishchuk
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from rest_framework import routers

from django_obm import views


def register_views(router):
    router.register(r"transactions", views.TransactionViewSet)
    router.register(r"currencies", views.CurrencyViewSet)
    router.register(r"addresses", views.AddressViewSet)
    return router


urlpatterns = register_views(routers.SimpleRouter()).urls



# Copyright 2019-2020 Alexander Polishchuk
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from django.conf import settings
from rest_framework import decorators, response, viewsets

from django_obm import models, pagination, serializers


def get_pagination_class():
    # TODO: Add pagination class setting
    # fmt: off
    has_pagination = getattr(settings, "OBM_PAGINATION_LIMIT", None) \
        or getattr(settings, "OBM_PAGINATION_MAX_LIMIT", None)
    # fmt: on
    return pagination.CustomLimitOffsetPagination if has_pagination else None


class TransactionViewSet(viewsets.ModelViewSet):
    """The ViewSet for work with transactions.  """

    serializer_class = serializers.TransactionSerializer
    queryset = models.Transaction.objects.all()
    pagination_class = get_pagination_class()


class AddressViewSet(viewsets.ModelViewSet):
    """The ViewSet for work with addresses.  """

    serializer_class = serializers.AddressSerializer
    queryset = models.Address.objects.all()
    pagination_class = get_pagination_class()


class CurrencyViewSet(viewsets.ModelViewSet):
    """The ViewSet for work with currencies.  """

    serializer_class = serializers.CurrencySerializer
    queryset = models.Currency.objects.all()

    # pylint: disable=unused-argument, no-self-use
    @decorators.action(detail=False, methods=["post"])
    def estimate_fee(self, request):
        serializer = serializers.CurrencyEstimateFeeSerializer(
            data=request.data
        )
        assert serializer.is_valid(raise_exception=True)
        currency = serializer.validated_data["currency"]
        node = currency.default_node
        estimated_fee = node.estimate_fee(
            from_address=request.query_params.get("from_address"),
            to_address=request.query_params.get("to_address"),
            amount=request.query_params.get("amount"),
            fee={
                "gas": request.query_params.get("gas"),
                "gas_price": request.query_params.get("gas_price"),
            },
            data=request.query_params.get("data"),
            conf_target=request.query_params.get("conf_target", 1),
        )
        node.close()
        return response.Response(
            {"currency": currency.name, "estimated_fee": estimated_fee,}
        )
